import glob
import numpy as np
import cv2
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
import os
import re
from scipy import signal
from scipy.optimize import curve_fit
plt.rcParams['pdf.fonttype'] = 42


def func(x, MIN, MAX, EC50, Hillslope):
    return MIN + (MAX - MIN)/(1 + (x/EC50)**(-Hillslope))


if __name__ == "__main__":
    cmap = sns.color_palette()
    cmap_hm = "plasma"

    CFPs = sorted(glob.glob("./_results_CFP*.csv"))
    FRETs = sorted(glob.glob("./_results_FRET*.csv"))
    df_data_to_plot = pd.DataFrame({})
    for k in range(len(CFPs)):
        df_CFP = pd.read_csv(CFPs[k], index_col=0)
        df_FRET = pd.read_csv(FRETs[k], index_col=0)
        pattern = r'([0-9]+\.?[0-9]*n)'
        conc = re.findall(pattern, CFPs[k])[0][:-1]
        if conc == "0":
            continue
        lot = CFPs[k][-7]
        ratio = [j for j in df_FRET["Mean"]/df_CFP["Mean"]]
        temp = pd.DataFrame([ratio, [float(conc) for j in range(len(ratio))], [int(lot) for j in range(len(ratio))]],
                            index=["ratio", "concentration", "lot"])
        df_data_to_plot = pd.concat([df_data_to_plot, temp.T], ignore_index=True)

    df_mean = df_data_to_plot.groupby(["concentration", "lot"], as_index=False).mean()
    print(df_mean)
    lots = df_mean["lot"].unique().tolist()  # first-durationを重複なく抜き出す

    fig = plt.figure(figsize=(8, 6))
    popt, pcov = curve_fit(func, xdata=df_mean["concentration"], ydata=df_mean["ratio"], p0=[1.3, 2.0, 1000, -2])
    print(popt)
    print([pcov[j][j]**0.5 for j in range(len(pcov[0]))])
    fit_x = np.linspace(min(df_mean["concentration"]), max(df_mean["concentration"]), 1000)
    fit_y = func(fit_x, *popt)
    sns.scatterplot(x="concentration", y="ratio", data=df_mean, hue="lot", s=100, palette="colorblind", legend=False)
    plt.plot(fit_x, fit_y, color="black")
    plt.xscale('log')
    plt.savefig("./__dose-response_fit.png")
    plt.savefig("./__dose-response_fit.pdf")
    plt.close(fig)
